/******************************************************************************
* BIM M 13x Serial Communication
*
* File: utility.c
*
* Date: 2009/03/16
*
* Version: 002
*
* Author: Schiekofer
******************************************************************************/
#include "BIM_M13x.h"
#include "utility.h"

extern const CObjPtr CObjects[];
typedef struct COMMSTABLE                       //!< com table structure
{
    BYTE ActualLength;
    BYTE pRAMFlags;
    BYTE ObjectDescrp[][3];
} COMMSTABLE;

COMMSTABLE* ptrCommunicationObjectTable;

const BYTE ComObjLengthTable[] = {0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
                                  0x02, 0x03, 0x04, 0x06, 0x08, 0x0A, 0x0E, 0x00};

void Utility_Init(void)
{
    // get pointer for CommunicationObjectTable
    ptrCommunicationObjectTable = (COMMSTABLE*)GetPointer(CommunicationObjectTable);
}

BYTE GetObjectLength(BYTE ObjNum)
{
    BYTE ObjType;
    
    // check if object number valid
    if (ObjNum < ptrCommunicationObjectTable->ActualLength)
    {
        // get the object type
        ObjType = ptrCommunicationObjectTable->ObjectDescrp[ObjNum][2];
        return ComObjLengthTable[ObjType & 0x0F];    
    }
    else
    {
        // invalid object number
        return 0;
    }
}

BYTE* GetObjectPointer(BYTE ObjNum)
{    
    // check if object number valid
    if (ObjNum < ptrCommunicationObjectTable->ActualLength)
    {
        return CObjects[ObjNum + 1];
    }
    else
    {
        // invalid object number
        return NULL;
    }
}

BYTE GetObjectType(BYTE ObjNum)
{    
    // check if object number valid
    if (ObjNum < ptrCommunicationObjectTable->ActualLength)
    {
        return ptrCommunicationObjectTable->ObjectDescrp[ObjNum][2];
    }
    else
    {
        // invalid object number        
        return 0xFF;
    }
}

// returns the pointer to the specified table
USHORT GetPointer(Table tab)
{
    MESSAGEQUEUE UserQueue = NULL;
    MESSAGE* pMsg;
    USHORT pointer = NULL;

    // check if tab has valid value
    if ((tab < AddressTable) ||
        (tab > CommunicationObjectTable))
    {
        return pointer;
    }
    
    // get a new message from system
    // (the memory space of the message is
    //  automatically set to zero)
    do
    {
        // release and do an immediate task switch 
        FORCE_TASKSWITCH();
        pMsg = U._MsgCreate();
    }while (pMsg == NULL);
    
    // create a property value read
    // (source and destination address
    //  will be zero)
    pMsg->Length = 0x0C;        
    pMsg->ServiceCode = T_DATA_ind;
    pMsg->Data[ 6] = A_PROPERTYVALUE_READ_HIBYTE; 
    pMsg->Data[ 7] = A_PROPERTYVALUE_READ_LOBYTE;
    pMsg->Data[ 8] = tab;                                       // object index
    pMsg->Data[ 9] = 0x07;                                      // property id
    pMsg->Data[10] = 0x10;                                      // high nibble: num of elements
    pMsg->Data[11] = 0x01;                                      // this + low nibble of previous byte: array index
    
    // block task switch
    DIS_TASKSWITCH();

    // get all messages that are posted to queue TransportLayerConReq
    U._MsgSwitchQueue(&U._pDynQueues->TransportLayerConReq, &UserQueue);

    // post message to ManagementLayer
    U._MsgPost(pMsg, &U._pStaticQueues->ManagementLayer);
    
    // catch the property value response
    for(;;)
    {
        do
        {
            // do an immediate task switch 
            FORCE_TASKSWITCH();
            // get message from TransportLayerConReq
            pMsg = U._MsgGet(&UserQueue);
        }while (pMsg == NULL);
        
        // check if it was our message
        if ((Word(pMsg->Data[1], pMsg->Data[2]) == 0x0000) &&                               // source address
            (Word(pMsg->Data[3], pMsg->Data[4]) == 0x0000) &&                               // destination address
            ((Word(pMsg->Data[6], pMsg->Data[7]) & APDUMASK) == A_PROPERTYVALUE_RESPONSE) ) // APDU
        {
            // get pointer
            pointer = Word(pMsg->Data[12], pMsg->Data[13]);
            pointer += 0x8000;
            // post message back to system
            U._MsgDiscard(pMsg);
            break;
        }
        else
        {
            // not our message -> post it to TransportLayerConReq
            U._MsgPost(pMsg, &U._pStaticQueues->TransportLayerConReq);
        }
    }
    // block task switch
    DIS_TASKSWITCH();   

    // restore the normal concatenation of the system message queues 
    U._MsgSwitchQueue(&U._pDynQueues->TransportLayerConReq, &U._pStaticQueues->TransportLayerConReq);

    // release task switch
    EN_TASKSWITCH();

    // post all messages from the UserQueue back to TransportLayerConReq
    do
    {
        pMsg = U._MsgGet(&UserQueue);
        if (pMsg != NULL)
        {
            U._MsgPost(pMsg, &U._pStaticQueues->TransportLayerConReq);
        }
    }while (pMsg != NULL);

    // return the pointer
    return pointer;
}
