using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace BIM_M_13x_SerialCom
{
    public partial class BIM_M_13x_SerialCom_Form : Form
    {
        private delegate void UpdateDataList(string s);
        #region PUBLIC_MEMBERS
        public BIM_M_13x_SerialCom_Form()
        {
            InitializeComponent();            
            m_updata = new UpdateDataList(UpdateComObjDataListFunc);
        }
        #endregion
        #region PRIVATE_MEMBERS
        private void UpdateComObjDataListFunc(string s)
        {                        
            LSB_Data.Items.Add(s);
            LBL_TelCount.Text = "Telegram count: " + LSB_Data.Items.Count.ToString();
        }
        private void BIM_M_13x_SerialCom_Form_Load(object sender, EventArgs e)
        {
            this.Text = "BIM M 13x SerialCom (v" + Application.ProductVersion + ")";

            CBX_SerialPort.SelectedIndex = 1;
            m_activetab = 0;

            // create a new instance of BIM_M_13x_Serial_ComObj
            m_bsc = new BIM_M_13x_Serial_ComObj();
            // get events
            m_bsc.ErrorHandler += new BIM_M_13x_SerialCom_LowLevel.OnReceiveError(m_bsc_ErrorHandler);
            m_bsc.OnComObjIndication += new BIM_M_13x_Serial_ComObj.OnComObjIndicationEventHandler(m_bsc_OnComObjIndication);
            m_bsc.OnComObjWriteConfirm += new BIM_M_13x_Serial_ComObj.OnComObjWriteConfirmEventHanler(m_bsc_OnComObjWriteConfirm);
            m_bsc.OnComObjReadConfirm += new BIM_M_13x_Serial_ComObj.OnComObjReadConfirmEventHanler(m_bsc_OnComObjReadConfirm);
            m_bsc.OnComObjValueReadResponse += new BIM_M_13x_Serial_ComObj.OnComObjValueReadResponseEventHandler(m_bsc_OnComObjValueReadResponse);
            m_bsc.OnComObjSetRamFlagsResponse += new BIM_M_13x_Serial_ComObj.OnComObjSetRamFlagsResponseEventHandler(m_bsc_OnComObjSetRamFlagsResponse);
            m_bsc.OnComObjGetTypeResponse += new BIM_M_13x_Serial_ComObj.OnComObjGetTypeResponseEventHandler(m_bsc_OnComObjGetTypeResponse);            

            // create a new instance of BIM_M_13x_Serial_GrpAddr
            m_bsg = new BIM_M_13x_Serial_GrpAddr();
            // get events
            m_bsg.OnGrpAddrWriteConfirm += new BIM_M_13x_Serial_GrpAddr.OnGrpAddrWriteConfirmEventHandler(m_bsg_OnGrpAddrWriteConfirm);
            m_bsg.OnGrpAddrWriteIndication += new BIM_M_13x_Serial_GrpAddr.OnGrpAddrWriteIndicationEventHandler(m_bsg_OnGrpAddrWriteIndication);
            m_bsg.OnGrpAddrReadConfirm += new BIM_M_13x_Serial_GrpAddr.OnGrpAddrReadConfirmEventHandler(m_bsg_OnGrpAddrReadConfirm);
            m_bsg.OnGrpAddrReadIndication += new BIM_M_13x_Serial_GrpAddr.OnGrpAddrReadIndicationEventHandler(m_bsg_OnGrpAddrReadIndication);
            m_bsg.OnGrpAddrResponseConfirm += new BIM_M_13x_Serial_GrpAddr.OnGrpAddrResponseConfirmEventHandler(m_bsg_OnGrpAddrResponseConfirm);
            m_bsg.OnGrpAddrResponseIndication += new BIM_M_13x_Serial_GrpAddr.OnGrpAddrResponseIndicationEventHandler(m_bsg_OnGrpAddrResponseIndication);
        }
        private void m_bsc_ErrorHandler()
        {
            MessageBox.Show("Communication error", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        private void m_bsc_OnComObjIndication(int ObjNum, byte[] data)
        {
            object[] para = new object[1];
            StringBuilder sb = new StringBuilder();

            foreach (byte b in data)
            {
                sb.Append("0x");
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0').ToUpper());
                sb.Append(' ');
            }

            sb.Remove(sb.Length - 1, 1);
            para[0] = "ComObj Indication (No. " + ObjNum.ToString() + "): " + sb.ToString();            
            Invoke(m_updata, para);            
        }
        private void m_bsc_OnComObjWriteConfirm(int ObjNum, bool confirm)
        {
            object[] para = new object[1];
            para[0] = "ComObj write confirm (No. " + ObjNum.ToString() + "): " + confirm.ToString();
            Invoke(m_updata, para);
        }
        private void m_bsc_OnComObjReadConfirm(int ObjNum, bool confirm)
        {
            object[] para = new object[1];
            para[0] = "ComObj read confirm (No. " + ObjNum.ToString() + "): " + confirm.ToString();
            Invoke(m_updata, para);
        }
        private void m_bsc_OnComObjValueReadResponse(int ObjNum, byte[] data)
        {
            object[] para = new object[1];
            StringBuilder sb = new StringBuilder();

            foreach (byte b in data)
            {
                sb.Append("0x");
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0').ToUpper());
                sb.Append(' ');
            }

            sb.Remove(sb.Length - 1, 1);
            para[0] = "ComObj value read response (No. " + ObjNum.ToString() + "): " + sb.ToString();
            Invoke(m_updata, para);
        }
        private void m_bsc_OnComObjSetRamFlagsResponse(int ObjNum, byte response)
        {
            string s;
            object[] para = new object[1];
            s = "0x" + Convert.ToString(response, 16).PadLeft(2, '0').ToUpper();
            para[0] = "ComObj set ram flags response (No. " + ObjNum.ToString() + "): " + s;
            Invoke(m_updata, para);
        }
        private void m_bsc_OnComObjGetTypeResponse(int ObjNum, byte type)
        {
            string s;
            object[] para = new object[1];
            s = "0x" + Convert.ToString(type, 16).PadLeft(2, '0').ToUpper();
            para[0] = "ComObj get type response (No. " + ObjNum.ToString() + "): " + s;
            Invoke(m_updata, para);
        }
        private void m_bsg_OnGrpAddrWriteConfirm(ushort GrpAddr, bool Less7Bit, byte[] data)
        {
            object[] para = new object[1];
            StringBuilder sb = new StringBuilder();

            sb.Append("GrpAddr Write Confirm (Addr. ");
            sb.Append(Convert.ToString(GrpAddr, 16).PadLeft(4, '0').ToUpper());
            sb.Append(", Less7Bit = ");
            sb.Append(Less7Bit.ToString());
            sb.Append("): ");

            foreach (byte b in data)
            {
                sb.Append("0x");
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0').ToUpper());
                sb.Append(' ');
            }
            sb.Remove(sb.Length - 1, 1);

            para[0] = sb.ToString();
            Invoke(m_updata, para);
        }
        private void m_bsg_OnGrpAddrWriteIndication(ushort GrpAddr, bool Less7Bit, byte[] data)
        {
            object[] para = new object[1];
            StringBuilder sb = new StringBuilder();

            sb.Append("GrpAddr Write Indication (Addr. ");
            sb.Append(Convert.ToString(GrpAddr, 16).PadLeft(4, '0').ToUpper());
            sb.Append(", Less7Bit = ");
            sb.Append(Less7Bit.ToString());
            sb.Append("): ");
            
            foreach (byte b in data)
            {
                sb.Append("0x");
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0').ToUpper());
                sb.Append(' ');
            }
            sb.Remove(sb.Length - 1, 1);

            para[0] = sb.ToString();
            Invoke(m_updata, para);
        }
        private void m_bsg_OnGrpAddrReadConfirm(ushort GrpAddr)
        {
            object[] para = new object[1];
            StringBuilder sb = new StringBuilder();

            sb.Append("GrpAddr Read Confirm (Addr. ");
            sb.Append(Convert.ToString(GrpAddr, 16).PadLeft(4, '0').ToUpper());
            sb.Append(")");
            para[0] = sb.ToString();
            Invoke(m_updata, para);            
        }
        private void m_bsg_OnGrpAddrReadIndication(ushort GrpAddr)
        {
            object[] para = new object[1];
            StringBuilder sb = new StringBuilder();

            sb.Append("GrpAddr Read Indication (Addr. ");
            sb.Append(Convert.ToString(GrpAddr, 16).PadLeft(4, '0').ToUpper());
            sb.Append(")");
            para[0] = sb.ToString();
            Invoke(m_updata, para);
        }
        private void m_bsg_OnGrpAddrResponseConfirm(ushort GrpAddr, bool Less7Bit, byte[] data)
        {
            object[] para = new object[1];
            StringBuilder sb = new StringBuilder();

            sb.Append("GrpAddr Response Confirm, (Addr. ");
            sb.Append(Convert.ToString(GrpAddr, 16).PadLeft(4, '0').ToUpper());
            sb.Append(", Less7Bit = ");
            sb.Append(Less7Bit.ToString());
            sb.Append("): ");

            foreach (byte b in data)
            {
                sb.Append("0x");
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0').ToUpper());
                sb.Append(' ');
            }
            sb.Remove(sb.Length - 1, 1);

            para[0] = sb.ToString();
            Invoke(m_updata, para);
        }
        private void m_bsg_OnGrpAddrResponseIndication(ushort GrpAddr, bool Less7Bit, byte[] data)
        {
            object[] para = new object[1];
            StringBuilder sb = new StringBuilder();

            sb.Append("GrpAddr Response Indication (Addr. ");
            sb.Append(Convert.ToString(GrpAddr, 16).PadLeft(4, '0').ToUpper());
            sb.Append(", Less7Bit = ");
            sb.Append(Less7Bit.ToString());
            sb.Append("): ");

            foreach (byte b in data)
            {
                sb.Append("0x");
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0').ToUpper());
                sb.Append(' ');
            }
            sb.Remove(sb.Length - 1, 1);

            para[0] = sb.ToString();
            Invoke(m_updata, para);
        }
        private void BTN_Connnect_Click(object sender, EventArgs e)
        {
            if (BTN_Connnect.Text == "Connect")
            {
                BIM_M_13x_SerialCom_LowLevel.BIM_M_13x_SerialCom_LowLevel_System system;
                if (RBTN_Windows.Checked)
                {
                    system = BIM_M_13x_SerialCom_LowLevel.BIM_M_13x_SerialCom_LowLevel_System.windows; 
                }
                else
                {
                    system = BIM_M_13x_SerialCom_LowLevel.BIM_M_13x_SerialCom_LowLevel_System.linux;
                }

                try
                {
                    if (tbc_SerialCom.SelectedTab.Text == "Communication objects")
                    {
                        m_bsc.Connect(int.Parse(CBX_SerialPort.Items[CBX_SerialPort.SelectedIndex].ToString()), system);
                    }
                    else
                    {
                        m_bsg.Connect(int.Parse(CBX_SerialPort.Items[CBX_SerialPort.SelectedIndex].ToString()), system);
                    }                    
                }
                catch (Exception)
                {
                    MessageBox.Show("Error, could not connect!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                BTN_Connnect.Text = "Disconnect";
                m_activetab = tbc_SerialCom.SelectedIndex;
            }
            else
            {
                try
                {
                    if (tbc_SerialCom.SelectedTab.Text == "Communication objects")
                    {
                        m_bsc.Disconnect();
                    }
                    else
                    {
                        m_bsg.Disconnect();
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Error, could not disconnect!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                BTN_Connnect.Text = "Connect";
            }
        }
        private void BTN_ClearLSBData_Click(object sender, EventArgs e)
        {
            LSB_Data.Items.Clear();
        }
        private void BIM_M_13x_SerialCom_Form_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_bsc.Disconnect();
            m_bsg.Disconnect();
        }
        private void BTN_ComObjWriteRequest_Click(object sender, EventArgs e)
        {
            string s;
            int ObjNum;

            try
            {
                ObjNum = int.Parse(TBX_ObjNumWR.Text);
            }
            catch (Exception)
            {
                MessageBox.Show("Error in ObjNum", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            s = TBX_SendData.Text.Replace(" ", "").Replace("0x", "").Trim();
            if (System.Text.RegularExpressions.Regex.Match(s, "^([A-Fa-f0-9]{2}){1,16}$").Success)
            {
                byte[] ba = new byte[s.Length / 2];
                for (int i = 0; i < ba.Length; i++)
                {
                    ba[i] = byte.Parse(s.Substring((i * 2), 2), System.Globalization.NumberStyles.HexNumber);
                }
                if (m_bsc.SendComObjWriteRequest(ObjNum, ba) == false)
                {
                    ShowErrorBox();
                }
            }
            else
            {
                MessageBox.Show("Error in data string", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void BTN_ComObjReadRequest_Click(object sender, EventArgs e)
        {
            int ObjNum;

            try
            {
                ObjNum = int.Parse(TBX_ObjNumRR.Text);
            }
            catch (Exception)
            {
                MessageBox.Show("Error in ObjNum", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (m_bsc.SendComObjReadRequest(ObjNum) == false)
            {
                ShowErrorBox();
            }
        }
        private void BTN_ComObjValueReadRequest_Click(object sender, EventArgs e)
        {
            int ObjNum;

            try
            {
                ObjNum = int.Parse(TBX_ObjNumVRR.Text);
            }
            catch (Exception)
            {
                MessageBox.Show("Error in ObjNum", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (m_bsc.SendComObjValueReadRequest(ObjNum) == false)
            {
                ShowErrorBox();
            }
        }
        private void BTN_ComObjSetRamFlags_Click(object sender, EventArgs e)
        {
            string s;
            int ObjNum;
            byte RamFlags;

            try
            {
                ObjNum = int.Parse(TBX_ObjNumSRF.Text);

            }
            catch (Exception)
            {
                MessageBox.Show("Error in ObjNum", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            s = TBX_FlagsByte.Text.Replace(" ", "").Replace("0x", "").Trim();
            if (System.Text.RegularExpressions.Regex.Match(s, "^([A-Fa-f0-9]{2}){1}$").Success)
            {                                
                RamFlags = byte.Parse(s, System.Globalization.NumberStyles.HexNumber);
                if (m_bsc.SendComObjSetRamFlags(ObjNum, RamFlags) == false)
                {
                    ShowErrorBox();
                }
            }
            else
            {
                MessageBox.Show("Error in flags byte", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }           
        }
        private void BTN_ComObjGetType_Click(object sender, EventArgs e)
        {
            int ObjNum;

            try
            {
                ObjNum = int.Parse(TBX_ObjNumGT.Text);
            }
            catch (Exception)
            {
                MessageBox.Show("Error in ObjNum", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (m_bsc.SendComObjGetType(ObjNum) == false)
            {
                ShowErrorBox();
            }
        }
        private void BTN_GrpAddrWriteRequest_Click(object sender, EventArgs e)
        {
            string s;
            ushort GrpAddr;

            try
            {
                GrpAddr = ushort.Parse(TBX_GrpAddrWR.Text, System.Globalization.NumberStyles.HexNumber);
            }
            catch (Exception)
            {
                MessageBox.Show("Error in GrpAddr", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            s = TBX_GrpAddrSendData.Text.Replace(" ", "").Replace("0x", "").Trim();
            if (System.Text.RegularExpressions.Regex.Match(s, "^([A-Fa-f0-9]{2}){1,16}$").Success)
            {
                byte[] ba = new byte[s.Length / 2];
                for (int i = 0; i < ba.Length; i++)
                {
                    ba[i] = byte.Parse(s.Substring((i * 2), 2), System.Globalization.NumberStyles.HexNumber);
                }
                if (m_bsg.SendGrpAddrWriteRequest(GrpAddr, CBX_Less7Bit_WR.Checked, ba) == false)
                {
                    ShowErrorBox();
                }
            }
            else
            {
                MessageBox.Show("Error in data string", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void BTN_GrpAddrReadRequest_Click(object sender, EventArgs e)
        {            
            ushort GrpAddr;

            try
            {
                GrpAddr = ushort.Parse(TBX_GrpAddrRR.Text, System.Globalization.NumberStyles.HexNumber);
            }
            catch (Exception)
            {
                MessageBox.Show("Error in GrpAddr", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            m_bsg.SendGrpAddrReadRequest(GrpAddr);
        }
        private void BTN_GrpAddrResponseRequest_Click(object sender, EventArgs e)
        {
            string s;
            ushort GrpAddr;

            try
            {
                GrpAddr = ushort.Parse(TBX_GrpAddrReR.Text, System.Globalization.NumberStyles.HexNumber);
            }
            catch (Exception)
            {
                MessageBox.Show("Error in GrpAddr", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            s = TBX_GrpAddrSendDataReR.Text.Replace(" ", "").Replace("0x", "").Trim();
            if (System.Text.RegularExpressions.Regex.Match(s, "^([A-Fa-f0-9]{2}){1,16}$").Success)
            {
                byte[] ba = new byte[s.Length / 2];
                for (int i = 0; i < ba.Length; i++)
                {
                    ba[i] = byte.Parse(s.Substring((i * 2), 2), System.Globalization.NumberStyles.HexNumber);
                }
                if (m_bsg.SendGrpAddrResponseRequest(GrpAddr, CBX_Less7Bit_ReR.Checked, ba) == false)
                {
                    ShowErrorBox();
                }
            }
            else
            {
                MessageBox.Show("Error in data string", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void tbc_SerialCom_MouseDown(object sender, MouseEventArgs e)
        {
            if (BTN_Connnect.Text != "Connect")
            {
                tbc_SerialCom.SelectedIndex = m_activetab;
            }
        }
        private void ShowErrorBox()
        {
            MessageBox.Show("Could not send", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        #endregion
        #region PRIVATE_VARIABLES
        private BIM_M_13x_Serial_ComObj m_bsc;
        private BIM_M_13x_Serial_GrpAddr m_bsg;
        private UpdateDataList m_updata;
        private int m_activetab;
        #endregion
    }
}