/******************************************************************************
* BIM M 13x Serial Communication
*
* File: BIM_M_13x_Serial_GrpAddr.cs
*
* Date: 2009/03/16
*
* Version: 002
*
* Author: Schiekofer
******************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;

namespace BIM_M_13x_SerialCom
{
    class BIM_M_13x_Serial_GrpAddr : BIM_M_13x_SerialCom_LowLevel
    {
        #region PUBLIC_EVENTS
        public delegate void OnGrpAddrWriteConfirmEventHandler(ushort GrpAddr, bool Less7Bit, byte[] data);
        public event BIM_M_13x_Serial_GrpAddr.OnGrpAddrWriteConfirmEventHandler OnGrpAddrWriteConfirm;
        public delegate void OnGrpAddrWriteIndicationEventHandler(ushort GrpAddr, bool Less7Bit, byte[] data);
        public event BIM_M_13x_Serial_GrpAddr.OnGrpAddrWriteIndicationEventHandler OnGrpAddrWriteIndication;
        public delegate void OnGrpAddrReadConfirmEventHandler(ushort GrpAddr);
        public event BIM_M_13x_Serial_GrpAddr.OnGrpAddrReadConfirmEventHandler OnGrpAddrReadConfirm;
        public delegate void OnGrpAddrReadIndicationEventHandler(ushort GrpAddr);
        public event BIM_M_13x_Serial_GrpAddr.OnGrpAddrReadIndicationEventHandler OnGrpAddrReadIndication;
        public delegate void OnGrpAddrResponseIndicationEventHandler(ushort GrpAddr, bool Less7Bit, byte[] data);
        public event BIM_M_13x_Serial_GrpAddr.OnGrpAddrResponseIndicationEventHandler OnGrpAddrResponseIndication;
        public delegate void OnGrpAddrResponseConfirmEventHandler(ushort GrpAddr, bool Less7Bit, byte[] data);
        public event BIM_M_13x_Serial_GrpAddr.OnGrpAddrResponseConfirmEventHandler OnGrpAddrResponseConfirm;
        #endregion
        #region PUBLIC_MEMBERS
        public bool SendGrpAddrWriteRequest(ushort GrpAddr, bool Less7Bit, byte[] data)
        {
            // generate a new message
            byte[] ba = new byte[data.Length + 4];
            // set command
            ba[MESSAGE_CMD_POS] = GRPADDR_WRITE_REQUEST;
            // set group address
            ba[MESSAGE_GRPADDR_POS_HIGH] = (byte)((GrpAddr >> 8) & 0xFF);
            ba[MESSAGE_GRPADDR_POS_LOW] = (byte)(GrpAddr & 0xFF);
            if (Less7Bit == true)
            {
                ba[MESSAGE_LESS7BIT_POS] = 0x01;
            }
            else
            {
                ba[MESSAGE_LESS7BIT_POS] = 0x00;
            }
            // copy payload into message
            Array.Copy(data, 0, ba, 4, data.Length);
            // send message
            return base.SendMsg(ba);
        }
        public bool SendGrpAddrReadRequest(ushort GrpAddr)
        {
            // generate a new message
            byte[] ba = new byte[3];
            // set command
            ba[MESSAGE_CMD_POS] = GRPADDR_READ_REQUEST;
            // set group address
            ba[MESSAGE_GRPADDR_POS_HIGH] = (byte)((GrpAddr >> 8) & 0xFF);
            ba[MESSAGE_GRPADDR_POS_LOW] = (byte)(GrpAddr & 0xFF);
            // send message
            return base.SendMsg(ba);
        }
        public bool SendGrpAddrResponseRequest(ushort GrpAddr, bool Less7Bit, byte[] data)
        {
            // generate a new message
            byte[] ba = new byte[data.Length + 4];
            // set command
            ba[MESSAGE_CMD_POS] = GRPADDR_RESPONSE_REQUEST;
            // set group address
            ba[MESSAGE_GRPADDR_POS_HIGH] = (byte)((GrpAddr >> 8) & 0xFF);
            ba[MESSAGE_GRPADDR_POS_LOW] = (byte)(GrpAddr & 0xFF);
            if (Less7Bit == true)
            {
                ba[MESSAGE_LESS7BIT_POS] = 0x01;
            }
            else
            {
                ba[MESSAGE_LESS7BIT_POS] = 0x00;
            }
            // copy payload into message
            Array.Copy(data, 0, ba, 4, data.Length);
            // send message
            return base.SendMsg(ba); 
        }
        #endregion
        #region PROTECTED_MEMBERS
        protected override void OnReceiveMsg(byte[] msg)
        {
            // check kind of message
            switch (msg[MESSAGE_CMD_POS])
            {
                case GRPADDR_WRITE_CONFIRM:
                    if (OnGrpAddrWriteConfirm != null)
                    {
                        // generate a new byte array with payload length
                        byte[] data = new byte[msg.Length - 4];
                        // copy payload
                        Array.Copy(msg, 4, data, 0, data.Length);
                        // call event
                        OnGrpAddrWriteConfirm((ushort)((msg[1] << 8) + msg[2]), ((msg[3] == 0) ? false : true), data);
                    }
                    break;
                case GRPADDR_WRITE_INDICATION:
                    if (OnGrpAddrWriteIndication != null)
                    {
                        // generate a new byte array with payload length
                        byte[] data = new byte[msg.Length - 4];
                        // copy payload
                        Array.Copy(msg, 4, data, 0, data.Length);
                        // call event
                        OnGrpAddrWriteIndication((ushort)((msg[1] << 8) + msg[2]), ((msg[3] == 0) ? false : true), data);
                    }
                    break;
                case GRPADDR_READ_CONFIRM:
                    if (OnGrpAddrReadConfirm != null)
                    {
                        // call event
                        OnGrpAddrReadConfirm((ushort)((msg[1] << 8) + msg[2]));
                    }
                    break;
                case GRPADDR_READ_INDICATION:
                    if (OnGrpAddrReadIndication != null)
                    {
                        // call event
                        OnGrpAddrReadIndication((ushort)((msg[1] << 8) + msg[2]));
                    }
                    break;
                case GRPADDR_RESPONSE_CONFIRM:
                    if (OnGrpAddrResponseConfirm != null)
                    {
                        // generate a new byte array with payload length
                        byte[] data = new byte[msg.Length - 4];
                        // copy payload
                        Array.Copy(msg, 4, data, 0, data.Length);
                        // call event
                        OnGrpAddrResponseConfirm((ushort)((msg[1] << 8) + msg[2]), ((msg[3] == 0) ? false : true), data);
                    }
                    break;
                case GRPADDR_RESPONSE_INDICATION:
                    if (OnGrpAddrResponseIndication != null)
                    {
                        // generate a new byte array with payload length
                        byte[] data = new byte[msg.Length - 4];
                        // copy payload
                        Array.Copy(msg, 4, data, 0, data.Length);
                        // call event
                        OnGrpAddrResponseIndication((ushort)((msg[1] << 8) + msg[2]), ((msg[3] == 0) ? false : true), data);
                    }
                    break;
                default:
                    break;
            }
        }
        #endregion
        #region PRIVATE_CONST_VALUES
        private const byte GRPADDR_WRITE_REQUEST          = 0xB1;
        private const byte GRPADDR_WRITE_CONFIRM          = 0xB2;
        private const byte GRPADDR_WRITE_INDICATION       = 0xB3;
        private const byte GRPADDR_READ_REQUEST           = 0xB5;
        private const byte GRPADDR_READ_CONFIRM           = 0xB6;
        private const byte GRPADDR_READ_INDICATION        = 0xB7;
        private const byte GRPADDR_RESPONSE_REQUEST       = 0xB9;
        private const byte GRPADDR_RESPONSE_CONFIRM       = 0xBA;
        private const byte GRPADDR_RESPONSE_INDICATION    = 0xBB;
        private const int MESSAGE_CMD_POS = 0;
        private const int MESSAGE_GRPADDR_POS_HIGH        =    1;
        private const int MESSAGE_GRPADDR_POS_LOW         =    2;
        private const int MESSAGE_LESS7BIT_POS            =    3;
        #endregion
    }
}
