/******************************************************************************
* BIM M 13x Serial Communication
*
* File: BIM_M_13x_Serial_GrpAddr.c
*
* Date: 2009/03/16
*
* Version: 002
*
* Author: Schiekofer
******************************************************************************/
#include "BIM_M13x.h"
#include "BIM_M_13x_Serial_GrpAddr.h"
#include "BIM_M_13x_Serial_LowLevel.h"

#define GRPADDR_WRITE_REQUEST         0xB1
#define GRPADDR_WRITE_CONFIRM         0xB2
#define GRPADDR_WRITE_INDICATION      0xB3
#define GRPADDR_READ_REQUEST          0xB5
#define GRPADDR_READ_CONFIRM          0xB6
#define GRPADDR_READ_INDICATION       0xB7
#define GRPADDR_RESPONSE_REQUEST      0xB9
#define GRPADDR_RESPONSE_CONFIRM      0xBA
#define GRPADDR_RESPONSE_INDICATION   0xBB


#pragma dataseg = EXPRAM
__no_init BYTE TempBuffer[17];
#pragma dataseg = default

void SerialGrpAddr_Init()
{
}

void SGA_WriteConfirm(USHORT GrpAddr, BYTE* pData, BYTE length)
{
    // set group address
    TempBuffer[0] = HIBYTE(GrpAddr);
    TempBuffer[1] = LOBYTE(GrpAddr);
    // indicate if data is Less7Bit
    if (length == 0)
    {
        TempBuffer[2] = 0x01;
        length = 4;
    }
    else
    {
        TempBuffer[2] = 0x00;
        length += 3;
    }
    // copy payload
    for (BYTE i = 3; i < length; i++)
    {
        TempBuffer[i] = *pData;
        pData++;
    }
    // send data
    SLLSend(TempBuffer, GRPADDR_WRITE_CONFIRM, length);    
}

void SGA_WriteIndication(USHORT GrpAddr, BYTE* pData, BYTE length)
{
    // set group address
    TempBuffer[0] = HIBYTE(GrpAddr);
    TempBuffer[1] = LOBYTE(GrpAddr);
    // indicate if data is Less7Bit
    if (length == 0)
    {
        TempBuffer[2] = 0x01;
        length = 4;
    }
    else
    {
        TempBuffer[2] = 0x00;
        length += 3;
    }
    // copy payload
    for (BYTE i = 3; i < length; i++)
    {
        TempBuffer[i] = *pData;
        pData++;
    }
    // send data
    SLLSend(TempBuffer, GRPADDR_WRITE_INDICATION, length);
}

void SGA_ReadConfirm(USHORT GrpAddr)
{
    // set group address
    TempBuffer[0] = HIBYTE(GrpAddr);
    TempBuffer[1] = LOBYTE(GrpAddr);
    // send data
    SLLSend(TempBuffer, GRPADDR_READ_CONFIRM, 2);    
}

void SGA_ReadIndication(USHORT GrpAddr)
{
    // set group address
    TempBuffer[0] = HIBYTE(GrpAddr);
    TempBuffer[1] = LOBYTE(GrpAddr);
    // send data
    SLLSend(TempBuffer, GRPADDR_READ_INDICATION, 2);    
}

void SGA_ResponseConfirm(USHORT GrpAddr, BYTE* pData, BYTE length)
{
    // set group address
    TempBuffer[0] = HIBYTE(GrpAddr);
    TempBuffer[1] = LOBYTE(GrpAddr);
    // indicate if data is Less7Bit
    if (length == 0)
    {
        TempBuffer[2] = 0x01;
        length = 4;
    }
    else
    {
        TempBuffer[2] = 0x00;
        length += 3;
    }
    // copy payload
    for (BYTE i = 3; i < length; i++)
    {
        TempBuffer[i] = *pData;
        pData++;
    }
    // send data
    SLLSend(TempBuffer, GRPADDR_RESPONSE_CONFIRM, length);
}

void SGA_ResponseIndication(USHORT GrpAddr, BYTE* pData, BYTE length)
{
    // set group address
    TempBuffer[0] = HIBYTE(GrpAddr);
    TempBuffer[1] = LOBYTE(GrpAddr);
    // indicate if data is Less7Bit
    if (length == 0)
    {
        TempBuffer[2] = 0x01;
        length = 4;
    }
    else
    {
        TempBuffer[2] = 0x00;
        length += 3;
    }
    // copy payload
    for (BYTE i = 3; i < length; i++)
    {
        TempBuffer[i] = *pData;
        pData++;
    }
    // send data
    SLLSend(TempBuffer, GRPADDR_RESPONSE_INDICATION, length);
}

void SLL_OnDataReceived(BYTE* data, BYTE Length)
{
    MESSAGE* pMsg;

    // try to get a free message from system
    pMsg = U._MsgCreate();    

    // check command byte
    switch (data[0])
    {
    case GRPADDR_WRITE_REQUEST:
        // check if message is valid
        if (pMsg != NULL)
        {
            // check if length is valid
            if (Length > (14 + 4))
            {
                U._MsgDiscard(pMsg);
                break;
            }
            // set service code
            pMsg->ServiceCode = L_GROUP_DATA_requ;

            // control byte
            pMsg->Data[0] = 0xBC;
            // destination address (source address is set by system)
            pMsg->Data[3] = data[1];
            pMsg->Data[4] = data[2];            
            pMsg->Data[6] = 0x00;

            // Less7Bit?
            if (data[3] != 0x00)
            {                
                pMsg->Length = 8;
                pMsg->Data[5] = 0xE1;
                pMsg->Data[7] = 0x80 | (data[4] & 0x3F);
            }
            else
            {
                pMsg->Length = 8 + Length - 4;
                pMsg->Data[5] = 0xE1 + (Length - 4);
                pMsg->Data[7] = 0x80;
                // copy payload
                for (BYTE i = 0; i < (Length - 4); i++)
                {
                    pMsg->Data[8 + i] = data[4 + i];
                }
            }
            // post message to system
            U._MsgPost(pMsg, &U._pStaticQueues->LinkLayer);
        }
        break;
    case GRPADDR_READ_REQUEST:
        // check if message is valid
        if (pMsg != NULL)
        {
            pMsg->Length = 8;
            pMsg->ServiceCode = L_GROUP_DATA_requ;

            // control byte
            pMsg->Data[0] = 0xBC;
            // destination address (source address is set by system)
            pMsg->Data[3] = data[1];
            pMsg->Data[4] = data[2];
            pMsg->Data[5] = 0xE1;
            pMsg->Data[6] = 0x00;
            pMsg->Data[7] = 0x00;
            // post message to system
            U._MsgPost(pMsg, &U._pStaticQueues->LinkLayer);
        }        
        break;
    case GRPADDR_RESPONSE_REQUEST:
        // check if message is valid
        if (pMsg != NULL)
        {
            // check if length is valid
            if (Length > (14 + 4))                
            {
                U._MsgDiscard(pMsg);
                break;
            }
            // set service code
            pMsg->ServiceCode = L_GROUP_DATA_requ;

            // set control byte
            pMsg->Data[0] = 0xBC;
            // destination address (source address is set by system)
            pMsg->Data[3] = data[1];
            pMsg->Data[4] = data[2];
            pMsg->Data[6] = 0x00;

            // Less7Bit?
            if (data[3] != 0x00)
            {                
                pMsg->Length = 8;
                pMsg->Data[5] = 0xE1;
                pMsg->Data[7] = 0x40 | (data[4] & 0x3F);
            }
            else
            {
                pMsg->Length = 8 + Length - 4;
                pMsg->Data[5] = 0xE1 + (Length - 4);
                pMsg->Data[7] = 0x40;
                for (BYTE i = 0; i < (Length - 4); i++)
                {
                    pMsg->Data[8 + i] = data[4 + i];
                }
            }
            // post message to system
            U._MsgPost(pMsg, &U._pStaticQueues->LinkLayer);
        }            
        break;
    default:        
        break;
    }
}
