/******************************************************************************
* BIM M 13x Serial Communication
*
* File: BIM_M_13x_Serial_ComObject.c
*
* Date: 2009/03/16
*
* Version: 002
*
* Author: Schiekofer
******************************************************************************/
#include "BIM_M13x.h"
#include "BIM_M_13x_Serial_ComObject.h"
#include "BIM_M_13x_Serial_LowLevel.h"
#include "utility.h"

#define COMMOBJ_INDICATION              0xA1
#define COMMOBJ_WRITEREQUEST            0xA2
#define COMMOBJ_WRITECONFIRM            0xA3
#define COMMOBJ_READREQUEST             0xA6
#define COMMOBJ_READCONFIRM             0xA7
#define COMMOBJ_VALUEREADREQUEST        0xA9
#define COMMOBJ_VALUEREADRESPONSE       0xAA
#define COMMOBJ_SETRAMFLAGS             0xAC
#define COMMOBJ_SETRAMFLAGSRESPONSE     0xAD
#define COMMOBJ_GETTYPE                 0xAE
#define COMMOBJ_GETTYPERESPONSE         0xAF

#pragma dataseg = EXPRAM
__no_init BYTE TempBuffer[15];
#pragma dataseg = default

void SerialComObject_Init()
{
}

BOOL SCO_Indication(BYTE* pData, BYTE Length)
{
    // send data with command COMMOBJ_INDICATION
    return SLLSend(pData, COMMOBJ_INDICATION, Length);    
}

void SLL_OnDataReceived(BYTE* data, BYTE Length)
{
    BYTE command = data[0];
    BYTE ObjNum =  data[1];
    
    // check what type of message was received
    switch (command)
    {
    case COMMOBJ_WRITEREQUEST:        
        {
            // check if not too much bytes are in the message
            if ((Length - 2) <= GetObjectLength(ObjNum))
            {
                // generate a confirm
                // with object number
                TempBuffer[0] = ObjNum;
                // get return value of the API function
                TempBuffer[1] = U._SetAndTransmitObject(ObjNum, &data[2], Length);        
                // send message
                SLLSend(TempBuffer, COMMOBJ_WRITECONFIRM, 2);
            }
            break;
        }
    case COMMOBJ_READREQUEST:
        {
            TempBuffer[0] = ObjNum;
            TempBuffer[1] = U._ReadObject(ObjNum);
            SLLSend(TempBuffer, COMMOBJ_READCONFIRM, 2);
            break;
        }
    case COMMOBJ_VALUEREADREQUEST:
        {
            BYTE ObjLength;
            BYTE* ObjPointer;
            
            ObjLength = GetObjectLength(ObjNum);
            // get pointer to object            
            ObjPointer = GetObjectPointer(ObjNum);
            if (ObjPointer != NULL)
            {
                for (BYTE i = 0; i < ObjLength; i++)
                {
                    // copy object data
                    TempBuffer[i + 1] = ObjPointer[i];
                }
                TempBuffer[0] = ObjNum;
                SLLSend(TempBuffer, COMMOBJ_VALUEREADRESPONSE, ObjLength + 1);
            }
            break;
        }
    case COMMOBJ_SETRAMFLAGS:
        {
            BYTE Flags;
            
            // just check if object number is valid
            if (GetObjectPointer(ObjNum) != NULL)
            {
                Flags = data[2];
                Flags = U._SetRAMFlags(ObjNum, Flags);
                TempBuffer[0] = ObjNum;
                TempBuffer[1] = Flags;
                SLLSend(TempBuffer, COMMOBJ_SETRAMFLAGSRESPONSE, 2);
            }
            break;
        }
    case COMMOBJ_GETTYPE:
        {
            BYTE type;
            
            type = GetObjectType(ObjNum);
            if (type != 0xFF)
            {
                TempBuffer[0] = ObjNum;
                TempBuffer[1] = type;
                SLLSend(TempBuffer, COMMOBJ_GETTYPERESPONSE, 2);
            }
            break;            
        }
    default:
        break;
    }
}
