/******************************************************************************
* BIM M 13x Serial Communication
*
* File: BIM_M_13x_Serial_LowLevel.h
*
* Date: 2009/03/16
*
* Version: 002
*
* Author: Schiekofer
******************************************************************************/
#ifndef __BIM_M_13X_SERIAL_LOWLEVEL_H
#define __BIM_M_13X_SERIAL_LOWLEVEL_H

#define SERIAL_MAX_MESSAGE_PAYLOAD    18
#define SERIAL_LL_ADDED_BYTES          4
#define SERIAL_MAX_MESSAGE_LENGTH   (SERIAL_MAX_MESSAGE_PAYLOAD + SERIAL_LL_ADDED_BYTES)
#define SERIAL_NUM_OF_BUFFERS       0x03

#define SERIAL_INC_BUFFERINDEX(i)        i++; if (i == SERIAL_NUM_OF_BUFFERS) i = 0;

#define SERIAL_START_SEND()             STIF0 = 1

#define SERIAL_ACK                  0xC3
#define SERIAL_NACK                 0xCC
#define ACK_MASK                    0xC0

#define SERIAL_POS_LENGTH           0x00
#define SERIAL_POS_SEQCOUNTER       0x01
#define SERIAL_POS_CMD              0x02

#define SERIAL_EN_RCVINT()          SRMK0 = 0
#define SERIAL_DIS_RCVINT()         SRMK0 = 1

#define SERIAL_ACK_TIMEOUT          MSEC100
#define SERIAL_ACK_TIMEOUT_SHORT     MSEC30
#define SERIAL_RCV_TIMEOUT           MSEC30

#define RESET   0x33

typedef struct
{
    BOOL process;
    BYTE count;
    BYTE Buffer[SERIAL_MAX_MESSAGE_LENGTH];
} BufferStruct;

typedef enum
{
    sll_trm_idle             = 0,
    sll_trm_transmitting     = 1,
    sll_trm_transmit_finish  = 2,
    sll_trm_waitforack       = 3,
} SLLtrmStates;

typedef enum
{
    sll_rcv_idle             = 0,
    sll_rcv_receiving        = 1,
    sll_rcv_waitend          = 2,
} SLLrcvStates;

typedef enum
{
    sll_tmctrl_idle         = 0,
    sll_tmctrl_on           = 1,
    sll_tmctrl_off          = 2,    
} SLLtimercontrol;

typedef enum
{
    sll_reset_idle          = 0,
    sll_reset_req           = 1,
    sll_reset_conf          = 2,   
} SLLResetState;

typedef struct
{
    BYTE* rcvPointer;
    const BYTE* trmPointer;
    BYTE rcvCount;
    BYTE trmCount;
    BYTE rcvIndex;
    BYTE rcvIndexProcess;
    BYTE trmIndex;
    BYTE rcvSeqCounter;
    BYTE trmSeqCounter;    
    BOOL rcvACK;
    BOOL trmACK;
    BYTE trmACKByte;
    SLLResetState reset;
    SLLrcvStates rcvState;
    SLLtrmStates trmState;
    SLLtimercontrol timerctrl;
} SerialLowLevelData;

void SerialLowLevel_Init();
void SerialLowLevel_Task();
BOOL SLLSend(const BYTE* ptr, BYTE cmd, BYTE length);

#endif
