/******************************************************************************
* BIM M 13x Serial Communication
*
* File: application.c
*
* Date: 2009/03/16
*
* Version: 002
*
* Author: Schiekofer
******************************************************************************/
#include "BIM_M13x.h"
#include "BIM_M_13x_Serial_LowLevel.h"
#include "BIM_M_13x_Serial_ComObject.h"
#include "utility.h"
#include "application.h"

BYTE ObjectDataBuffer[15];
TIMER_TAB UserTimerTab;

void AppInit(void)
{
    // clear EXPRAM
    for (USHORT x = 0xF000; x < 0xF400; x++)
    {
        *(BYTE*)x = 0x00;
    }

    // init timers    
    U._TmInit(NUM_OF_TIMERS);
    // init serial communication modules
    SerialLowLevel_Init();
    SerialComObject_Init();
    
    // after initialization a taskswitch must be done because
    // the task switch is disabled at cstartup
    FORCE_TASKSWITCH();
    
    // init utility
    Utility_Init();    
}

void TestAndSendObject(BYTE ObjNum)
{
    BYTE ObjLength;
    BYTE *pObjPtr;
    
    if (U._TestObject(ObjNum) != FALSE)
    {
        DIS_TASKSWITCH();
        // first get length of requested object
        ObjLength = GetObjectLength(ObjNum);
        pObjPtr = GetObjectPointer(ObjNum);
    
        // test object and copy object data on update
        for (BYTE i = 0; i < ObjLength; i++)
        {
            ObjectDataBuffer[i + 1] = *pObjPtr;
            pObjPtr++;
        }
        // add object number
        ObjectDataBuffer[0] = ObjNum;
        // send data
        SCO_Indication(ObjectDataBuffer, (ObjLength + 1));    
        EN_TASKSWITCH();
    }
}


void Application_Task(void)
{
    // loop over all communication objects and send data on update
    for (BYTE i = 0; i < NUM_OF_COM_OBJ; i++)
    {
        TestAndSendObject(i);
        // do the serial task
        if (i%16 == 0)
        {
            SerialLowLevel_Task();
        }
    }                
}
