/******************************************************************************
* BIM M 13x Serial Communication
*
* File: BIM_M_13x_Serial_ComObj.cs
*
* Date: 2009/03/16
*
* Version: 002
*
* Author: Schiekofer
******************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;

namespace BIM_M_13x_SerialCom
{
    class BIM_M_13x_Serial_ComObj : BIM_M_13x_SerialCom_LowLevel
    {
        #region PUBLIC_EVENTS
        public delegate void OnComObjIndicationEventHandler(int ObjNum, byte[] data);
        public event BIM_M_13x_Serial_ComObj.OnComObjIndicationEventHandler OnComObjIndication;
        public delegate void OnComObjWriteConfirmEventHanler(int ObjNum, bool confirm);
        public event BIM_M_13x_Serial_ComObj.OnComObjWriteConfirmEventHanler OnComObjWriteConfirm;
        public delegate void OnComObjReadConfirmEventHanler(int ObjNum, bool confirm);
        public event BIM_M_13x_Serial_ComObj.OnComObjReadConfirmEventHanler OnComObjReadConfirm;
        public delegate void OnComObjValueReadResponseEventHandler(int ObjNum, byte[] data);
        public event BIM_M_13x_Serial_ComObj.OnComObjValueReadResponseEventHandler OnComObjValueReadResponse;
        public delegate void OnComObjSetRamFlagsResponseEventHandler(int ObjNum, byte response);
        public event BIM_M_13x_Serial_ComObj.OnComObjSetRamFlagsResponseEventHandler OnComObjSetRamFlagsResponse;
        public delegate void OnComObjGetTypeResponseEventHandler(int ObjNum, byte type);
        public event BIM_M_13x_Serial_ComObj.OnComObjGetTypeResponseEventHandler OnComObjGetTypeResponse;
        #endregion
        #region PUBLIC_MEMBERS
        public bool SendComObjWriteRequest(int ObjNum, byte[] data)
        {
            // generate a new message
            byte[] ba = new byte[data.Length + 2];            
            // set command
            ba[MESSAGE_CMD_POS] = COMMOBJ_WRITEREQUEST;
            // set object number
            ba[MESSAGE_OBJNUM_POS] = (byte)(ObjNum & 0xFF);
            // copy payload into message
            Array.Copy(data, 0, ba, 2, data.Length);
            // send message
            return base.SendMsg(ba);
        }
        public bool SendComObjReadRequest(int ObjNum)
        {
            byte[] ba = new byte[2];
            ba[MESSAGE_CMD_POS] = COMMOBJ_READREQUEST;
            ba[MESSAGE_OBJNUM_POS] = (byte)(ObjNum & 0xFF);
            return base.SendMsg(ba);
        }
        public bool SendComObjValueReadRequest(int ObjNum)
        {
            byte[] ba = new byte[2];
            ba[MESSAGE_CMD_POS] = COMMOBJ_VALUEREADREQUEST;
            ba[MESSAGE_OBJNUM_POS] = (byte)(ObjNum & 0xFF);
            return base.SendMsg(ba);        
        }
        public bool SendComObjSetRamFlags(int ObjNum, byte ramflags)
        {
            byte[] ba = new byte[3];
            ba[MESSAGE_CMD_POS] = COMMOBJ_SETRAMFLAGS;
            ba[MESSAGE_OBJNUM_POS] = (byte)(ObjNum & 0xFF);
            ba[2] = ramflags;
            return base.SendMsg(ba);
        }
        public bool SendComObjGetType(int ObjNum)
        {
            byte[] ba = new byte[2];
            ba[MESSAGE_CMD_POS] = COMMOBJ_GETTYPE;
            ba[MESSAGE_OBJNUM_POS] = (byte)(ObjNum & 0xFF);
            return base.SendMsg(ba);       
        }
        #endregion
        #region PROTECTED_MEMBERS
        protected override void OnReceiveMsg(byte[] msg)
        {
            // check kind of message
            switch (msg[0])
            {
                case COMMOBJ_INDICATION:
                    // first check if user of this class wants to get the event
                    if (OnComObjIndication != null)
                    {
                        // generate a new byte array with payload length
                        byte[] data = new byte[msg.Length - 2];
                        // copy payload
                        Array.Copy(msg, 2, data, 0, data.Length);
                        // call event
                        OnComObjIndication(msg[1], data);
                    }
                    break;
                case COMMOBJ_WRITECONFIRM:
                    if (OnComObjWriteConfirm != null)
                    {
                        bool confirm;

                        confirm = (msg[2] == 0x01) ? true : false;
                        OnComObjWriteConfirm(msg[1], confirm);
                    }
                    break;
                case COMMOBJ_READCONFIRM:
                    if (OnComObjReadConfirm != null)
                    {
                        bool confirm;

                        confirm = (msg[2] == 0x01) ? true : false;
                        OnComObjReadConfirm(msg[1], confirm);
                    }
                    break;
                case COMMOBJ_VALUEREADRESPONSE:
                    if (OnComObjValueReadResponse != null)
                    {
                        byte[] data = new byte[msg.Length - 2];
                        Array.Copy(msg, 2, data, 0, data.Length);
                        OnComObjValueReadResponse(msg[1], data);
                    }
                    break;
                case COMMOBJ_SETRAMFLAGSRESPONSE:
                    if (OnComObjSetRamFlagsResponse != null)
                    {
                        OnComObjSetRamFlagsResponse(msg[1], msg[2]);
                    }
                    break;
                case COMMOBJ_GETTYPERESPONSE:
                    if (OnComObjGetTypeResponse != null)
                    {
                        OnComObjGetTypeResponse(msg[1], msg[2]);
                    }
                    break;
                default:
                    break;
            }
        }
        #endregion
        #region PRIVATE_CONST_VALUES
        private const byte COMMOBJ_INDICATION           = 0xA1;
        private const byte COMMOBJ_WRITEREQUEST         = 0xA2;    
        private const byte COMMOBJ_WRITECONFIRM         = 0xA3;
        private const byte COMMOBJ_READREQUEST          = 0xA6;
        private const byte COMMOBJ_READCONFIRM          = 0xA7;
        private const byte COMMOBJ_VALUEREADREQUEST     = 0xA9;
        private const byte COMMOBJ_VALUEREADRESPONSE    = 0xAA;
        private const byte COMMOBJ_SETRAMFLAGS          = 0xAC;
        private const byte COMMOBJ_SETRAMFLAGSRESPONSE  = 0xAD;
        private const byte COMMOBJ_GETTYPE              = 0xAE;
        private const byte COMMOBJ_GETTYPERESPONSE      = 0xAF;       
        private const int  MESSAGE_CMD_POS              =    0;
        private const int  MESSAGE_OBJNUM_POS           =    1;
        #endregion
    }
}
